using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.IO.Ports;

namespace Eq3.LCP100.Demo
{

  /// <summary>
  /// Hauptfenster der Demo-Anwendung
  /// </summary>
  public partial class MainForm : Form
  {
    /// <summary>
    /// maximale Anzahl der Bilder
    /// </summary>
    private const int INDEX_COUNT = 64;

    /// <summary>
    /// Baud-Rate (serieller Port)
    /// </summary>
    private const int BAUD_RATE = 38400;

    /// <summary>
    /// Daten-Bits (serieller Port)
    /// </summary>
    private const int DATA_BITS = 8;

    /// <summary>
    /// Paritt (serieller Port)
    /// </summary>
    private const Parity PARITY = Parity.Even;

    /// <summary>
    /// Stopp-Bits (serieller Port)
    /// </summary>
    private const StopBits STOP_BITS = StopBits.One;

    /// <summary>
    /// Lese-Timeout (serieller Port)exit
    /// </summary>
    private const int READ_TIMEOUT = 10000; // 10 Sekunden


    /// <summary>
    /// Seitengre
    /// </summary>
    private const int PAGE_SIZE = 512;

    /// <summary>
    /// Anzahl der 512-Byte-Seiten pro Bild
    /// </summary>
    private const int PAGES_PER_IMAGE = 64;

    /// <summary>
    /// LCP100
    /// </summary>
    private Eq3.LCP100.LCP100 lcp100;

    /// <summary>
    /// Gibt an, ob momentan eine Verbindung besteht
    /// </summary>
    private bool isConnected;

    /// <summary>
    /// Serieller Port
    /// </summary>
    private SerialPort serialPort;

    /// <summary>
    /// Erstellt das Hauptfenster
    /// </summary>
    public MainForm()
    {
      InitializeComponent();

      isConnected = false;

      // PortsComboBox
      String[] portNames = SerialPort.GetPortNames();
      PortsComboBox.Items.Clear();
      if (0 < portNames.Length)
      {
        PortsComboBox.Items.AddRange(portNames);
        PortsComboBox.SelectedItem = PortsComboBox.Items[0];
      }
      else
      {
        ConnectButton.Enabled = false;
        Disable();
      }

      // IndexComboBox
      IndexComboBox.Items.Clear();
      for (byte b = 0; b < INDEX_COUNT; b++)
      {
        IndexComboBox.Items.Add(b);
      }
      IndexComboBox.SelectedItem = IndexComboBox.Items[0];

      Disable();
    }

    /// <summary>
    /// Speichert ein Bild
    /// </summary>
    /// <param name="index">index des Bildes</param>
    /// <param name="image">Bildpuffer (32 KByte)</param>
    private void SavePicture(byte index, byte[] image)
    {
      try
      {
        int offset = index * PAGES_PER_IMAGE;
        for (int i = 0; i < PAGES_PER_IMAGE; i++)
        {
          byte[] page = new byte[PAGE_SIZE];
          for (int j = 0; j < PAGE_SIZE; j++) { page[j] = image[j + (i * PAGE_SIZE)]; }
          if (!lcp100.SavePage(offset + i, page)) { throw new Exception(); }
          lcp100.ShowPage(offset + i, PAGE_SIZE);
        }
      }
      catch
      {
        MessageBox.Show("Das Bild konnte nicht gespeichert werden.", "Bild speichern");
      }
    }

    /// <summary>
    /// Deaktiviert die Steuerelemente
    /// </summary>
    private void Disable()
    {
      ConnectionLabel.Text = "- nicht verbunden -";
      VersionLabel.Text = "n.a.";
      IndexComboBox.Enabled = false;
      LoadPictureButton.Enabled = false;
      SaveButton.Enabled = false;
      ShowTestImageButton.Enabled = false;
      SwitchBackLightOnButton.Enabled = false;
      SwitchBacklightOffButton.Enabled = false;
    }

    /// <summary>
    /// Aktiviert die Steuerelemente
    /// </summary>
    private void Enable()
    {
      IndexComboBox.Enabled = true;
      LoadPictureButton.Enabled = true;
      SaveButton.Enabled = true;
      ShowTestImageButton.Enabled = true;
      SwitchBackLightOnButton.Enabled = true;
      SwitchBacklightOffButton.Enabled = true;
    }

    /// <summary>
    /// Aktualisiert die Version
    /// </summary>
    /// <param name="version"></param>
    private void UpdateVersion(Eq3.LCP100.LCP100.Version version)
    {
      VersionLabel.Text = version.Major.ToString() + "." + version.Minor.ToString();
    }

    /// <summary>
    /// Baut eine Verbindung auf.
    /// </summary>
    private void Connect()
    {
      try
      {
        String portName = PortsComboBox.Text;
        serialPort = new SerialPort(portName, BAUD_RATE, 
          PARITY, DATA_BITS, STOP_BITS);
        serialPort.Open();
        Stream stream = serialPort.BaseStream;
        stream.ReadTimeout = READ_TIMEOUT;
        lcp100 = new Eq3.LCP100.LCP100(stream);
        UpdateVersion(lcp100.GetVersion());
        ConnectionLabel.Text = "verbunden mit " + portName;
        ConnectButton.Text = "Trennen";
        Enable();
        isConnected = true;
        IndexComboBox.SelectedItem = IndexComboBox.Items[0];
      }
      catch (Exception)
      {
        MessageBox.Show("Es konnte keine Verbindung aufgebaut werden.",
          "Verbinden");
        if(serialPort.IsOpen) serialPort.Close();
      }
    }

    /// <summary>
    /// Schliet die aktuelle Verbindung.
    /// </summary>
    private void Disconnect()
    {
      serialPort.Close();
      ConnectionLabel.Text = "- nicht verbunden -";
      ConnectButton.Text = "Verbinden";
      Disable();
      isConnected = false;
    }

    /// <summary>
    /// Zeigt ein Bild an
    /// </summary>
    /// <param name="index"></param>
    private void ShowImage(int index)
    {
      lcp100.SendCommand(0x2C);     // Display zum Schreiben vorbereiten

      for(int i = 0; i < PAGES_PER_IMAGE; i++)
      {
        int pageNumber = (index * PAGES_PER_IMAGE) + i;
        lcp100.ShowPage(pageNumber, PAGE_SIZE);
      }
    }

    /// <summary>
    /// ffnet bzw. Schliet die Verbindung.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ConnectButton_Click(object sender, EventArgs e)
    {
      if (!isConnected) { Connect(); }
      else              { Disconnect(); }
    }

    /// <summary>
    /// Zeigt das ausgewhlte Bild auf dem Display an.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void IndexComboBox_SelectedIndexChanged(object sender, EventArgs e)
    {
      if (isConnected)
      {
        ShowImage((byte)IndexComboBox.SelectedItem);
      }
    }

    /// <summary>
    /// Ldt das Vorschau-Bild.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LoadPictureButton_Click(object sender, EventArgs e)
    {
      if (DialogResult.OK == OpenFileDialog.ShowDialog())
      {
        try
        {
          PictureBox.Image = new Bitmap(OpenFileDialog.FileName);
        }
        catch (Exception)
        {
          MessageBox.Show("Fehler beim Laden des Bildes", "Bilder laden...");
        }
      }
    }

    /// <summary>
    /// Speichert das Vorschau-Bild im LCP100.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void SaveButton_Click(object sender, EventArgs e)
    {
      byte index = (byte) IndexComboBox.SelectedItem;
      Bitmap bitmap = new Bitmap(PictureBox.Image, 128, 128);
      byte[] buffer = new byte[2 * 128 * 128];
      int i = 0;
      for(int y = 0; y < 128; y++)
      {
        for(int x = 0; x < 128; x++)
        {
          Color pixel = bitmap.GetPixel(x, y);

          byte r = (byte)(((31 * pixel.R) / 255) & 0x1F);
          byte g = (byte)(((63 * pixel.G) / 255) & 0x3F);
          byte b = (byte)(((31 * pixel.B) / 255) & 0x1F);
         
          buffer[i++] = (byte)  ((r << 3) | (g >> 3));
          buffer[i++] = (byte)  ((g << 5) | b );

        }
      }
      SavePicture(index, buffer);
    }

    /// <summary>
    /// Zeigt das Testbild auf dem LCP100 an.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ShowTestImageButton_Click(object sender, EventArgs e)
    {
      lcp100.ShowTestImage();
    }

    /// <summary>
    /// Schaltet die Hintergrundbeleuchtung ein.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void SwitchBackLightOnButton_Click(object sender, EventArgs e)
    {
      lcp100.SwitchBacklightOn();
    }

    /// <summary>
    /// Schaltet die Hintergrundbeleuchtung aus.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void SwitchBacklightOffButton_Click(object sender, EventArgs e)
    {
      lcp100.SwitchBacklightOff();
    }

  }
}